/*****************************************************************************

    Name: fuzzy.h
    Description: fuzzy search library - header file
    
                 
    Author: Oleg Khovayko,  http://olegh.spedia.net

   ***************************************************************************

                          PUBLIC DOMAIN NOTICE
              National Center for Biotechnology Information

    This software/database is a "United States Government Work" under the
    terms of the United States Copyright Act.  It was written as part of    
    the author's official duties as a United States Government employee
    and thus cannot be copyrighted.  This software/database is freely
    available to the public for use. The National Library of Medicine and
    the U.S. Government have not placed any restriction on its use or
    reproduction.

    Although all reasonable efforts have been taken to ensure the accuracy
    and reliability of the software and data, the NLM and the U.S.
    Government do not and cannot warrant the performance or results that
    may be obtained by using this software or data. The NLM and the U.S.
    Government disclaim all warranties, express or implied, including
    warranties of performance, merchantability or fitness for any
    particular purpose.

    Please cite the author in any work or product based on this material.

   ***************************************************************************

    Entry Points:



*****************************************************************************/
#ifndef __fuzzyh__
#define __fuzzyh__ fuzzyh

#define Uint1  unsigned char
#define Uint2  unsigned short
#define Uint4  unsigned int
#define Int4   int

#define R50    Uint2

#define EOLN   0
#define SP     ' '

#define FALSE  0
#define TRUE   1

#define MAXR50 64000
#define MAXTERMLEN 128

#define BITS_FOR_LW 5
#define QTY_MASK ((1 << BITS_FOR_LW) - 1)
#define GET_TERM_NO(x) ((x) >> BITS_FOR_LW)
#define GET_TERM_LW(x) ((x) & QTY_MASK)

// #define CMP_TERM_N0_EQ(x, y) (((x)^(y)) <= QTY_MASK)

#define BITS_FOR_TN (32 - BITS_FOR_LW)
#define MAX_TN ((1 << BITS_FOR_TN) - 1)

typedef struct {
    R50    triade_r50;
    Uint2  qty;
} TRIADE_QTY;


typedef struct {
    float global_weight;
    Int4  offset, size;
    Uint2 seg_no, pos;
} XRS_ENTRY;

typedef struct fuzzy_merge {
  struct fuzzy_merge *next;
  Uint4  *ptr;
  Uint4  qty;
  float  weight;
  R50    triade;
} FUZZY_MERGE;

typedef struct fuzzy_found {
  Int4 term_no;
  float weight;
} FUZZY_FOUND;

typedef struct fuzzy_RESULT {
  float rate;
  Int4 term_no;
  const char *term;
  Uint2  term_len;
} FUZZY_RESULT;

typedef struct term_ENTRY {
  Int4 offset;
  float len_corr;
} TERM_ENTRY;


typedef struct fdb_HANDLE {
  XRS_ENTRY *statmas_entry;
  Uint4  *xrs_table;

  const char *termlist;
  const TERM_ENTRY *termlist_entry;

  const char *db_name, *path;
  struct fdb_HANDLE *next;
} FDB_HANDLE;



/* FILE: radix50.c */
extern R50 triade_to_r50(const char *str);
extern void r50_to_triade(R50 val, char *p);


/* FILE: triade.c */
void cpy_to_canonic(const char *instr, char *outstr);
Uint2 calculate_triades(const char *str, TRIADE_QTY *qty_buf, int mul_prefix);
Uint2 calculate_noncanonic_triades(const char *str, TRIADE_QTY *qty_buf, int mul_prefix);

/* FILE: weight.c */
extern const Uint1 count_to_lw[];
extern const Uint1 lw_to_count[];

/* FILE: find.c */

int fuzzy_find(FDB_HANDLE *h, const char *str, FUZZY_RESULT *out, int limit);

/* FILE: init.c */
int fuzzy_setup(FDB_HANDLE *h, const char *db_name, const char *path);
int fuzzy_init(FDB_HANDLE *h);


#endif
